@@ -20,7 +20,7 @@ residuals.mcemGLMM <- function(object, type = c("deviance", "pearson"), ...) {
     }
     if (type[1] == "deviance") {
       # print(mu0)
-      res0 <- sign(object$y - mu0) * sqrt(2 * object$y * log(object$y / mu0) - 2 * (object$y - mu0))
+      res0 <- sign(object$y - mu0) * sqrt(2 * ifelse(object$y > 0, object$y * log(object$y / mu0), 0) - 2 * (object$y - mu0))
     }
   }
   
@@ -31,7 +31,18 @@ residuals.mcemGLMM <- function(object, type = c("deviance", "pearson"), ...) {
       res0 <- (object$y - mu0) / sqrt(mu0 * (1 + 1/a0))
     }
     if (type[1] == "deviance") {
-      res0 <- sign(object$y - mu0) * sqrt(2 * (object$y * log(object$y / mu0)) - (object$y + a0) * log((object$y + a0)/(mu0 + a0)))
+      res0 <- sign(object$y - mu0) * sqrt(2 * (ifelse(object$y > 0, object$y * log(object$y / mu0), 0)) - 2 * (object$y + a0) * log((object$y + a0)/(mu0 + a0)))
+    }
+  }
+  
+  if (object$call$family == "gamma") {
+    mu0 <- exp(lin0)
+    a0 <- tail(object$mcemEST, 1)[kP + 1]
+    if (type[1] == "pearson") {
+      res0 <- (object$y - mu0) / (mu0 / sqrt(a0))
+    }
+    if (type[1] == "deviance") {
+      res0 <- sign(object$y - mu0) * sqrt(-2 * a0 * (log(object$y/mu0) - (object$y - mu0)/mu0))
     }
   }
   return(as.vector(res0))
